@@ -1,19 +1,22 @@
-// MProcess class - Installer-specific extensions to QProcess.
-//
-//   Copyright (C) 2019 by AK-47
-//   Licensed under the Apache License, Version 2.0 (the "License");
-//   you may not use this file except in compliance with the License.
-//   You may obtain a copy of the License at
-//
-//       http://www.apache.org/licenses/LICENSE-2.0
-//
-//   Unless required by applicable law or agreed to in writing, software
-//   distributed under the License is distributed on an "AS IS" BASIS,
-//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-//   See the License for the specific language governing permissions and
-//   limitations under the License.
-//
-// This file is part of the gazelle-installer.
+/***************************************************************************
+ * MProcess class - Installer-specific extensions to QProcess.
+ ***************************************************************************
+ *
+ *   Copyright (C) 2019 by AK-47
+ *   Licensed under the Apache License, Version 2.0 (the "License");
+ *   you may not use this file except in compliance with the License.
+ *   You may obtain a copy of the License at
+ *
+ *       http://www.apache.org/licenses/LICENSE-2.0
+ *
+ *   Unless required by applicable law or agreed to in writing, software
+ *   distributed under the License is distributed on an "AS IS" BASIS,
+ *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ *   See the License for the specific language governing permissions and
+ *   limitations under the License.
+ *
+ * This file is part of the gazelle-installer.
+ ***************************************************************************/
 
 #include <QApplication>
 #include <QDebug>
@@ -28,10 +31,10 @@ MProcess::MProcess(QObject *parent)
 {
 }
 
-void MProcess::setupUI(QListWidget *listLog, QProgressBar *progressBar)
+void MProcess::setupUI(QListWidget *listLog, QProgressBar *progInstall)
 {
     logView = listLog;
-    progBar = progressBar;
+    progBar = progInstall;
     QPalette pal = listLog->palette();
     pal.setColor(QPalette::Base, Qt::black);
     pal.setColor(QPalette::Text, Qt::white);
@@ -42,12 +45,16 @@ bool MProcess::exec(const QString &cmd, const bool rawexec, const QByteArray *in
 {
     if (halting) return false;
     ++execount;
-    qDebug().nospace() << "Exec #" << execount << ": " << cmd;
+    qDebug().nospace().noquote() << "Exec #" << execount << ": " << cmd;
     QListWidgetItem *logEntry = log(cmd, Exec);
     QEventLoop eloop;
     connect(this, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), &eloop, &QEventLoop::quit);
-    if (rawexec) start(cmd);
-    else start("/bin/bash", {"-c", cmd});
+    if (!rawexec) start("/bin/bash", {"-c", cmd});
+    else {
+        QStringList args = splitCommand(cmd);
+        QString prog = args.takeFirst();
+        start(prog, args);
+    }
     if (!debugUnusedOutput) {
         if (!needRead) closeReadChannel(QProcess::StandardOutput);
         closeReadChannel(QProcess::StandardError);
@@ -108,30 +115,47 @@ void MProcess::halt()
 void MProcess::unhalt()
 {
     QEventLoop eloop;
-    connect(this, static_cast<void(QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished), &eloop, &QEventLoop::quit);
+    connect(this, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), &eloop, &QEventLoop::quit);
     if (state() != QProcess::NotRunning) {
         closeReadChannel(QProcess::StandardOutput);
         closeReadChannel(QProcess::StandardError);
         closeWriteChannel();
         eloop.exec();
     }
-    disconnect(this, static_cast<void(QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished), nullptr, nullptr);
+    disconnect(this, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), nullptr, nullptr);
     halting = false;
 }
 
+QString MProcess::joinCommand(const QString &program, const QStringList &arguments)
+{
+    QString text = program;
+    for(QString arg : arguments) {
+        bool wspace = false;
+        for(const auto &ch : arg) {
+            if (ch.isSpace()) {
+                wspace = true;
+                break;
+            }
+        }
+        arg.replace("\"", "\\\"");
+        if (!wspace) text += ' ' + arg;
+        else text += " \"" + arg + "\"";
+    }
+    return text;
+}
+
 QListWidgetItem *MProcess::log(const QString &text, const LogType type)
 {
     if (type == Standard) qDebug().noquote() << text;
-    if (type == Section) qDebug().noquote() << "+++" << text << "+++";
-    else if (type == Status) qDebug().noquote() << "-" << text << "-";
-    if (!logView) return nullptr;
+    else if (type == Section) qDebug().noquote() << "<<" << text << ">>";
+    else if (type == Status) qDebug().noquote() << "++" << text << "++";
+    if (!logView || type == Section) return nullptr;
     QListWidgetItem *entry = new QListWidgetItem(text, logView);
     logView->addItem(entry);
     if (type == Exec) entry->setForeground(Qt::cyan);
-    else if (type != Standard) {
+    else if (type == Status) {
         QFont font(entry->font());
-        if (type == Section) font.setBold(true);
-        else if (type == Status) font.setItalic(true);
+        font.setBold(true);
         entry->setFont(font);
     }
     logView->scrollToBottom();
@@ -164,7 +188,7 @@ void MProcess::status(long progress)
     int slice = progSliceSpace;
     if (progSliceSteps > 0) {
         if (progSlicePos > progSliceSteps) progSlicePos = progSliceSteps;
-        slice = (progSlicePos * progSliceSpace) / progSliceSteps;
+        slice = static_cast<int>((progSlicePos * progSliceSpace) / progSliceSteps);
     }
     if (progBar) progBar->setValue(progSliceStart + slice);
     qApp->processEvents();
@@ -198,14 +222,14 @@ void MProcess::sleep(const int msec, const bool silent)
     const int rc = eloop.exec();
     if (!silent) {
         qDebug().nospace() << "Sleep #" << sleepcount << ": exit " << rc;
-        log(logEntry, rc==0?1:-1);
+        log(logEntry, rc == 0 ? 1 : -1);
     }
 }
 bool MProcess::mkpath(const QString &path)
 {
     QListWidgetItem *logEntry = log("MKPATH: "+path, Exec);
     const bool rc = QDir().mkpath(path);
-    qDebug() << (rc?"MkPath(SUCCESS):":"MkPath(FAILURE):") << path;
-    log(logEntry, rc?1:-1);
+    qDebug() << (rc ? "MkPath(SUCCESS):" : "MkPath(FAILURE):") << path;
+    log(logEntry, rc ? 1 : -1);
     return rc;
 }
